import { Modal, ModalOpts, KV } from './modal';
import { SelectEl, querySelector, elAddCS, ElStyle, elAddCS2 } from '../../utils/query-el';
import { ObjCss } from '../../utils/css';

export class BlankModal extends Modal {
  constructor(opts: ModalOpts) {
    super(opts);
  }
}
type DialogModalOpts = {
  dialogTitle?: string;
  dialogTitleClassName?: string;
  dialogTitleStyle?: ElStyle;
  dialogTitleEl?: HTMLElement;
  dialogTitleCloseIcon?: boolean;

  dialogContentClassName?: string;
  dialogContentStyle?: ElStyle;

  dialogFooterClassName?: string;
  dialogFooterStyle?: ElStyle;
} & ModalOpts<KV, DialogDefaultStyle>;

type DialogDefaultStyle = {
  defaultDialogTitleStyle?: ObjCss;
  defaultDialogContentStyle?: ObjCss;
  defaultDialogFooterStyle?: ObjCss;
};

export class DialogModal extends Modal<DialogModalOpts, DialogDefaultStyle> {
  dialogTitle?: string;
  dialogTitleClassName?: string;
  dialogTitleStyle?: ElStyle;
  dialogTitleEl?: HTMLElement;
  dialogTitleCloseIcon?: boolean;

  dialogContentClassName?: string;
  dialogContentStyle?: ElStyle;

  dialogFooterShow?: boolean;
  dialogFooterClassName?: string;
  dialogFooterStyle?: ElStyle;

  constructor(opts: ModalOpts<DialogModalOpts, DialogDefaultStyle>) {
    super(opts);

    this.dialogTitle = opts.dialogTitle;
    this.dialogTitleClassName = opts.dialogTitleClassName;
    this.dialogTitleStyle = opts.dialogTitleStyle;
    this.dialogTitleEl = opts.dialogTitleEl;
    this.dialogTitleCloseIcon = opts.dialogTitleCloseIcon;

    this.dialogContentClassName = opts.dialogContentClassName;
    this.dialogContentStyle = opts.dialogContentStyle;

    this.dialogFooterClassName = opts.dialogFooterClassName;
    this.dialogFooterStyle = opts.dialogFooterStyle;
    this.dialogFooterShow = opts.dialogFooterShow ?? false;

    this.setDefaultStyle('defaultContentStyle', {
      position: 'absolute',
      padding: '0px',
      left: '50%',
      top: '20%',
      width: '600px',
      background: '#fff',
      borderRadius: '5px',
      boxShadow: '0 0 10px rgba(0,0,0,.1)',
      transform: 'translate(-50%, -50%)',
      maxHeight: '80vh',
      overflow: 'auto',
      ...opts?.defaultStyle?.defaultContentStyle,
    });

    this.setDefaultStyle('defaultDialogTitleStyle', {
      display: 'flex',
      justifyContent: 'space-between',
      alignItems: 'center',
      position: 'sticky',
      padding: '10px 20px',
      top: '0',
      fontSize: '16px',
      background: '#fff',
      marginTop: '-10px',
      borderBottom: '1px solid #f0f0f0',
      marginBottom: '5px',
      ...opts?.defaultStyle?.defaultDialogTitleStyle,
    });
    this.setDefaultStyle('defaultDialogContentStyle', {
      padding: '20px',
      ...opts?.defaultStyle?.defaultDialogContentStyle,
    });
    this.setDefaultStyle('defaultDialogFooterStyle', {
      display: 'flex',
      justifyItems: 'end',
      borderTop: '1px solid #f0f0f0',
      padding: '10px 20px',
      ...opts?.defaultStyle?.defaultDialogFooterStyle,
    });
  }
  // static render(el: string | HTMLDivElement, id: string, opts?: DialogModalOpts): DialogModal;
  // static render(el: string | HTMLDivElement, opts?: DialogModalOpts): DialogModal;
  // static render(...args: any[]) {
  //   const [el, id, opts] = args;
  //   return super.render(el, id, opts);
  // }
  appendRoot(documentFragment: DocumentFragment): void {
    const cacheFragment = document.createDocumentFragment();
    // 拿出来
    cacheFragment.appendChild(this.Element);
    const DialogBox = documentFragment.querySelector('.ui-modal-content');
    DialogBox.classList.add('ui-modal-dialog');

    // 创建title
    const title = document.createElement('div');
    title.classList.add('ui-modal-dialog-title');
    elAddCS2(title, this.dialogTitleClassName, this.dialogTitleStyle, this.defaultStyle.defaultDialogTitleStyle);
    if (this.dialogTitleEl) {
      title.appendChild(this.dialogTitleEl);
    } else {
      title.innerText = this.dialogTitle;
    }
    if (this.dialogTitleCloseIcon) {
      const closeIcon = document.createElement('span');
      closeIcon.className = 'ui-modal-dialog-close';
      closeIcon.innerHTML = '&times;';
      closeIcon.style.cssText = `
        cursor: pointer;
        font-size: 24px;
        margin: -5px 10px 0 0;
      `;
      closeIcon.onclick = (e) => {
        this.setOpen(false);
      };
      title.appendChild(closeIcon);
    }

    DialogBox.appendChild(title);
    // 创建content
    const content = document.createElement('div');
    content.className = 'ui-modal-dialog-content';
    elAddCS2(content, this.dialogContentClassName, this.dialogContentStyle, this.defaultStyle.defaultDialogContentStyle);

    content.appendChild(cacheFragment);

    if (this.dialogFooterShow) {
      const footer = document.createElement('div');
      footer.className = 'ui-modal-dialog-footer';
      elAddCS2(footer, this.dialogFooterClassName, this.dialogFooterStyle, this.defaultStyle.defaultDialogFooterStyle);
      DialogBox.appendChild(footer);
    }

    DialogBox.appendChild(content);

    super.appendRoot(documentFragment);
  }
  renderEl(el: HTMLDivElement): this {
    return super.renderEl(el);
  }
  renderFooter(el?: SelectEl): void {
    const _el = querySelector(el);
    if (!_el) return;
    const footer = this.Element.querySelector('.ui-modal-dialog-footer');
    if (!footer) return;
    footer.innerHTML = '';
    footer.appendChild(_el);
  }
  renerContent(el?: SelectEl): void {
    const _el = querySelector(el);
    if (!_el) return;
    const content = this.Element.querySelector('.ui-modal-dialog-content');
    if (!content) return;
    content.innerHTML = '';
    content.appendChild(_el);
  }
}
